#
# Makefile Template for Tessy
# Compiler: GNU Tools for ARM
# Debugger: SEGGER JLink GDB for ARM
# Template Version: 2
#

# Defines to be generated by Tessy:
#
# BSP_DIRECTORYECTORY   :=
# BUFFER_SIZE           :=
# COMPILER_DEFINES      :=
# COMPILER_INCLUDES     :=
# COMPILER_INSTALL_PATH :=
# COMPILER_DEFINES      :=
# COMPILER_INCLUDES     :=
# COMPILER_INSTALL_PATH :=
# ENDIANESS             :=
# EXEFILE_EXTENSION     :=
# LINKER_FILE           :=
# MODULE_PATH           :=
# MODULE_PATH_DOS       :=
# OBJFILE_EXTENSION     :=
# PROJECTROOT           :=
# TARGET_INSTALL_PATH   :=
# TESSY_BIN             :=
# TESSY_SYS             :=
# TESTOBJECT            :=
# TIMER_FILE            :=
# TARGET_INSTALL_PATH   :=
# TIMER_FILE            :=
# USER_LINK_OPTIONS     :=

#####################################################################
#BEGIN_DEFINES
#####################################################################

#
# Special macros for includes and defines
#
TS_DEF_SEPARATOR :=-D
define TS_DEFINE
$(subst ;, $(TS_DEF_SEPARATOR),$(1))
endef

TS_INC_SEPARATOR :=-I
define TS_INCLUDE
$(subst ;, $(TS_INC_SEPARATOR),$(1))
endef

#
# Special macros for the linker's arguments file
#
ARGS_FILE := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s.args

define TS_LONGPATH
"$(shell ts_pathname -w '$(1)' | sed 's/\\\\/\\//g')"
endef

define TS_WRITE_LINE
echo $(call TS_LONGPATH,$(1),$(ARGS_FILE)) >> "$(ARGS_FILE)"

endef

define TS_WRITE_LINKER_FILE
$(foreach var,$(1),$(call TS_WRITE_LINE,$(var)))
endef

#
# Short pathes
#
TESSY_SYS_DOS   := $(shell ts_pathname -s $(TESSY_SYS))
GNU_TOOLS_HOME  := $(shell ts_pathname -s '$(COMPILER_INSTALL_PATH)')
MODULE_PATH_DOS := $(shell ts_pathname -s '$(MODULE_PATH_DOS)')
MODULE_PATH     := $(shell ts_pathname -s '$(MODULE_PATH)')

TS_NAME         := TS_GNUTOOLS
IDE_NAME        := codewarrior10
COMPANY_NAME    := gnutools
PROCESSOR_NAME  := arm

ifeq ("little","$(ENDIANESS)")
  TS_BYTEORDER := TS_HOST_BYTEORDER
  TS_BYTEORDER_OPTION :=
endif
ifeq ("big","$(ENDIANESS)")
  TS_BYTEORDER := TS_NET_BYTEORDER
  TS_BYTEORDER_OPTION :=
endif
ifndef TS_BYTEORDER
  $(error Attribute 'Endianess' has invalid value '$(ENDIANESS)', only 'little' and 'big' are allowed.)
endif

all	 : master slave
master : $(MODULE_PATH)\ts_$(TESTOBJECT)_m.exe
slave  : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(EXEFILE_EXTENSION)

#
# MASTER & COMM Makefiles
# (you should not need to modify these settings)
#
M_COMM_MAKE		:=	$(TESSY_SYS)/make/master_$(IDE_NAME)_gnu_i386.mk
S_COMM_MAKE		:=	$(TESSY_SYS)/src/comm/comm_$(IDE_NAME)_$(COMPANY_NAME)_$(PROCESSOR_NAME).mk


#
#
# SLAVE
#
S_CC           := $(shell ts_pathname -s '$(GNU_TOOLS_HOME)')\bin\arm-none-eabi-gcc.exe
S_LINK         := $(shell ts_pathname -s '$(GNU_TOOLS_HOME)')\bin\arm-none-eabi-gcc.exe
S_INCLUDES     := -I"$(TESSY_SYS_DOS)\include\tessy\comm" -I"$(MODULE_PATH_DOS)"
S_STUB_OBJECT  := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs$(OBJFILE_EXTENSION)
S_UC_OBJECT    := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr$(OBJFILE_EXTENSION)
# Tested CPU = cortex-m0
S_COMP_OPTIONS := -c -mcpu=$(CPU) -mthumb -mfloat-abi=soft -g3 -O0 -gdwarf-2 -gstrict-dwarf -Wall \
						-I"$(INITSRCDIR)" \
						-I"$(GNU_TOOLS_HOME)\..\..\MCU\ARM_GCC_Support\ewl\EWL_C\include" \
						-I"$(GNU_TOOLS_HOME)\..\..\MCU\ARM_GCC_Support\ewl\EWL_Runtime\include" \
						-I"$(GNU_TOOLS_HOME)\..\..\MCU\ProcessorExpert\lib\Kinetis\pdd\inc" \
						-I"$(GNU_TOOLS_HOME)\..\..\MCU\ProcessorExpert\lib\Kinetis\iofiles" \
						-O0 -ffunction-sections -fdata-sections -Wall -std=c99 -fmessage-length=0 \
						-specs=ewl_c.specs $(call TS_DEFINE,$(COMPILER_DEFINES)) -DTESSY -DTS_SLAVE \
					 	-D$(TS_NAME) -DTS_HAVE_INT64 -DTS_HAVE_FLOAT64
#					 	-MMD -MP -D$(TS_NAME) -DTS_HAVE_INT64 -DTS_HAVE_FLOAT64
S_ASM_OPTIONS  := -c -mthumb -mcpu=$(CPU) -mfloat-abi=soft -g3 -gdwarf-2 -gstrict-dwarf \
						-x assembler-with-cpp -Wall -fmessage-length=0
S_LINK_OPTIONS := -mcpu=$(CPU) -mthumb -mfloat-abi=soft -g3 -gdwarf-2 -gstrict-dwarf \
						-T"$(LINKER_FILE)" -Xlinker --gc-sections \
						-L"$(GNU_TOOLS_HOME)\..\..\MCU\ARM_GCC_Support\ewl\lib\armv6-m"
						-n -specs=ewl_c.specs -Xlinker --undefined=__pformatter_ \
						-Xlinker --defsym=__pformatter=__pformatter_ -Xlinker --undefined=__sformatter \
						-Xlinker --defsym=__sformatter=__sformatter

#####################################################################
#END_DEFINES
#####################################################################

#####################################################################
#BEGIN_SOURCE
#####################################################################

#
# Template rules
#
$(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER)$(OBJFILE_EXTENSION): $(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER).c
	$(S_CC) $(S_INCLUDES) $(call TS_INCLUDE,$(USER_INCLUDES$$(SOURCENUMBER))) $(S_COMP_OPTIONS) \
		$(call TS_DEFINE,$(USER_DEFINES$$(SOURCENUMBER))$(S_COM_OPTIONS)) \
		$(USER_COMP_OPTIONS$$(SOURCENUMBER)) -o "$@" "$<"


#####################################################################
#END_SOURCE
#####################################################################

#####################################################################
#BEGIN_TARGETS
#####################################################################

# COMPILE USERCODE / STUBS
#
$(S_UC_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr.c
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o "$@" "$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr.c"

$(S_STUB_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs.c
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o "$@" "$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs.c"


#
# COMPILE/LINK MASTER
#

include $(M_COMM_MAKE)


#
# COMPILE SLAVE COMMUNICATION MODULES

include $(S_COMM_MAKE)


#
#
# COMPILE/LINK SLAVE
#
$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s.c
	@echo *
	@echo ***** Compiling Slave *****
	$(S_CC) $(S_COMP_OPTIONS) $(S_INCLUDES) -o "$@" "$<"

#
# Handle Board Files
#
ifeq ("true","$(USE_BOARD_FILES)")
ifeq ("","$(INITOBJDIR)")
$(error Attribute 'Use Board Files' is set to true but 'InitObjDir' is empty.)
endif
ifeq ("","$(INITSRCDIR)")
$(error Attribute 'Use Board Files' is set to true but 'InitSrcDir' is empty.)
endif
INITOBJDIR := $(shell ts_pathname -s '$(INITOBJDIR)')
INITSRCDIR := $(shell ts_pathname -s '$(INITSRCDIR)')
ARM_OBJS   := $(patsubst $(INITSRCDIR)/%.c,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.c)) \
              $(patsubst $(INITSRCDIR)/%.S,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.S))
ARM_HDRS   := $(wildcard $(INITSRCDIR)/*.h*)

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.c $(ARM_HDRS)
	$(S_CC) $(S_COMP_OPTIONS) $(S_INCLUDES) -I$(INITSRCDIR) -c "$<" -o "$@"

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.S
	$(S_CC) $(S_ASM_OPTIONS) $(S_INCLUDES) -I$(INITSRCDIR) -c "$<" -o "$@"
endif

$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(EXEFILE_EXTENSION) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) \
															$(S_SRC_OBJECTS) $(ARM_OBJS) \
															$(S_UC_OBJECT) \
															$(S_STUB_OBJECT) \
															$(S_OBJECTS)
	@echo *
	@echo ***** Linking Slave *****
	@-ts_rm -f $(ARGS_FILE)
ifneq ($(ARM_OBJS),)
	@$(call TS_WRITE_LINKER_FILE,$(ARM_OBJS))
endif
	@$(call TS_WRITE_LINKER_FILE,$(S_OBJECTS))
	@$(call TS_WRITE_LINKER_FILE,$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION))
	@$(call TS_WRITE_LINKER_FILE,$(S_SRC_OBJECTS))
	@$(call TS_WRITE_LINKER_FILE,$(S_UC_OBJECT))
	@$(call TS_WRITE_LINKER_FILE,$(S_STUB_OBJECT))
	$(S_LINK) $(S_LINK_OPTIONS) \
				 $(USER_LINK_OPTIONS) \
				 -Wl,-Map="$(MODULE_PATH_DOS)\ts_$(TESTOBJECT).map" \
				 @"$(ARGS_FILE)" -o "$@" "$(LIBRARY)"
	@-ts_rm "$(MODULE_PATH)"/*$(OBJFILE_EXTENSION)

#####################################################################
#END_TARGETS
#####################################################################

